/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier1;

import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_3956;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.util.RebornInventory;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;
import techreborn.utils.RecipeUtils;
import techreborn.init.TRBlockEntities;

public class ElectricFurnaceBlockEntity extends PowerAcceptorBlockEntity
		implements IToolDrop, InventoryProvider, IContainerProvider {

	public RebornInventory<ElectricFurnaceBlockEntity> inventory = new RebornInventory<>(3, "ElectricFurnaceBlockEntity", 64, this);
	public int progress;
	public int fuelScale = 100;
	public int cost = 4;
	int input1 = 0;
	int output = 1;
	boolean wasBurning = false;

	public ElectricFurnaceBlockEntity() {
		super(TRBlockEntities.ELECTRIC_FURNACE );
	}

	public int gaugeProgressScaled(int scale) {
		return progress * scale / (int) (fuelScale * (1.0 - getSpeedMultiplier()));
	}

	public void cookItems() {
		if (canSmelt()) {
			final class_1799 itemstack = RecipeUtils.getMatchingRecipes(field_11863, class_3956.field_17546, inventory.method_5438(input1));

			if (inventory.method_5438(output).method_7960()) {
				inventory.method_5447(output, itemstack.method_7972());
			} else if (inventory.method_5438(output).method_7962(itemstack)) {
				inventory.method_5438(output).method_7933(itemstack.method_7947());
			}
			if (inventory.method_5438(input1).method_7947() > 1) {
				inventory.shrinkSlot(input1, 1);
			} else {
				inventory.method_5447(input1, class_1799.field_8037);
			}
		}
	}

	public boolean canSmelt() {
		if (inventory.method_5438(input1).method_7960()) {
			return false;
		}
		final class_1799 itemstack = RecipeUtils.getMatchingRecipes(field_11863, class_3956.field_17546, inventory.method_5438(input1));
		if (itemstack.method_7960()) {
			return false;
		}
		if (inventory.method_5438(output).method_7960()) {
			return true;
		}
		if (!inventory.method_5438(output).method_7962(itemstack)) {
			return false;
		}
		final int result = inventory.method_5438(output).method_7947() + itemstack.method_7947();
		return result <= this.inventory.getStackLimit() && result <= itemstack.method_7914();
	}

	public boolean isBurning() {
		return getEnergy() > getEuPerTick(cost);
	}

	public class_1799 getResultFor(class_1799 stack) {
		final class_1799 result = RecipeUtils.getMatchingRecipes(field_11863, class_3956.field_17546, stack);
		if (!result.method_7960()) {
			return result.method_7972();
		}
		return class_1799.field_8037;
	}

	public void updateState() {
		if (wasBurning != (progress > 0)) {
			// skips updating the block state for 1 tick, to prevent the machine from
			// turning on/off rapidly causing fps drops
			if (wasBurning && progress == 0 && canSmelt()) {
				wasBurning = true;
				return;
			}
			final class_2680 BlockStateContainer = field_11863.method_8320(field_11867);
			if (BlockStateContainer.method_11614() instanceof BlockMachineBase) {
				final BlockMachineBase blockMachineBase = (BlockMachineBase) BlockStateContainer.method_11614();
				if (BlockStateContainer.method_11654(BlockMachineBase.ACTIVE) != progress > 0)
					blockMachineBase.setActive(progress > 0, field_11863, field_11867);
			}
			wasBurning = (progress > 0);
		}

	}

	public int getBurnTime() {
		return progress;
	}

	public void setBurnTime(final int burnTime) {
		this.progress = burnTime;
	}

	// TilePowerAcceptor
	@Override
	public void method_16896() {
		super.method_16896();
		charge(2);

		if (field_11863.field_9236) {
			return;
		}

		final boolean burning = isBurning();
		boolean updateInventory = false;
		if (isBurning() && canSmelt()) {
			updateState();
			if (canUseEnergy(getEuPerTick(cost))) {
				useEnergy(getEuPerTick(cost));
				progress++;
				if (progress >= Math.max((int) (fuelScale * (1.0 - getSpeedMultiplier())), 5)) {
					progress = 0;
					cookItems();
					updateInventory = true;
				}
			}
		} else {
			updateState();
		}
		if (burning != isBurning()) {
			updateInventory = true;
		}
		if (updateInventory) {
			method_5431();
		}
	}

	@Override
	public double getBaseMaxPower() {
		return TechRebornConfig.electricFurnaceMaxEnergy;
	}

	@Override
	public boolean canAcceptEnergy(final class_2350 direction) {
		return true;
	}

	@Override
	public boolean canProvideEnergy(final class_2350 direction) {
		return false;
	}

	@Override
	public double getBaseMaxOutput() {
		return 0;
	}

	@Override
	public double getBaseMaxInput() {
		return TechRebornConfig.electricFurnaceMaxInput;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(final class_1657 entityPlayer) {
		return TRContent.Machine.ELECTRIC_FURNACE.getStack();
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory<ElectricFurnaceBlockEntity> getInventory() {
		return inventory;
	}

	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("electricfurnace").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this).slot(0, 55, 45).outputSlot(1, 101, 45).energySlot(2, 8, 72).syncEnergyValue()
				.syncIntegerValue(this::getBurnTime, this::setBurnTime).addInventory().create(this, syncID);
	}
}
